# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function
import sys
import json
import os
import requests
import six


def get_exp_info():
    """Returns the task info: {"type": "worker|master|ps", "index": int}."""
    info = os.getenv('AUTOCNN_EXP_INFO', None)
    try:
        return json.loads(info) if info else None
    except (ValueError, TypeError):
        print('Could get job info, '
              'please make sure this is running inside a autocnn job.')
        return None


def get_api(version='v1'):
    api = os.getenv('SEETAAS_HELPER_API', None)
    if not api:
        print('Could get api info, '
              'please make sure this is running inside a autocnn job.')
        return None
    return '{}'.format(api)

def get_autocnn_parameter():
    autocnn_parameter = os.getenv('AUTOCNN_PARAMETER', None)
    try:
        return json.loads(autocnn_parameter) if autocnn_parameter else None
    except (ValueError, TypeError):
        print('Could get autocnn parameter, '
              'please make sure this is running inside a autocnn job.')
        return None
    

def send_metrics(**metrics):
    """Sends metrics to seetaas helper api.

    Example:
        send_metric(precision=0.9, accuracy=0.89, loss=0.01)
    """
    exp_info = get_exp_info()
    exp_uuid = exp_info.get('exp_uuid', None)
    parameter_info = get_autocnn_parameter()
    #dataset_id = parameter.get('DEDUCTION_DATASET_ID', None)
    #dir_list_id = parameter.get('DIR_LIST_ID', None)
    label_dataset_id = os.getenv('DEDUCTION_DATASET_ID', None)
    origin_dataset_id = os.getenv('ORIGIN_DATASET_ID', None)
    dir_list_id = os.getenv('DIR_LIST_ID', None)
    api = get_api()
    if not all([exp_uuid, label_dataset_id, origin_dataset_id, dir_list_id, api]):
        print('Environment information not found, '
              'please make sure this is running inside a autocnn job.')
        return

    try:
        formatted_metrics = {k: v for k, v in six.iteritems(metrics)}
    except (ValueError, TypeError):
        print('Could not send metrics {}'.format(metrics))
        return
    try:
        requests.post('{}/api/helper/metrics/deduction/{}'.format(api, exp_uuid),
                      data={'label_dataset_id': label_dataset_id, 'origin_dataset_id': origin_dataset_id, 'dir_list_id': dir_list_id,'values': json.dumps(formatted_metrics)})
    except requests.RequestException as e:
        print('Could not reach autocnn api {}'.format(e))



from __future__ import unicode_literals
import json
import sys
import multiprocessing

import gunicorn.app.base
from gunicorn.six import iteritems

from cgi import parse_qs, escape

class Mapper:
    __mapper_relation = {}

    @staticmethod
    def register(cls, value):
        Mapper.__mapper_relation[cls] = value

    @staticmethod
    def exist(cls):
        if cls in Mapper.__mapper_relation:
            return True
        return False

    @staticmethod
    def get_value(cls):
        return Mapper.__mapper_relation[cls]


class AutoFill(type):
    def __call__(cls, *args, **kwargs):
        obj = cls.__new__(cls, *args, **kwargs)
        arg_list = list(args)
        if Mapper.exist(cls):
            value = Mapper.get_value(cls)
            arg_list.append(value)
        obj.__init__(*arg_list, **kwargs)
        return obj


class StandaloneApplication(gunicorn.app.base.BaseApplication):

    def __init__(self, app, options=None):
        self.options = options or {}
        self.application = app
        super(StandaloneApplication, self).__init__()

    def load_config(self):
        config = dict([(key, value) for key, value in iteritems(self.options)
                       if key in self.cfg.settings and value is not None])
        for key, value in iteritems(config):
            self.cfg.set(key.lower(), value)

    def load(self):
        return self.application

def response_404(environ, start_response):
   status = '404'

   response_json = json.dumps(dict(code=404, err_msg='Not found'))
   response_bytes = bytes(response_json, encoding = 'utf-8') 

   response_headers = [('Content-Type', 'text/html'),
                  ('Content-Length', str(len(response_json)))]
   
   start_response(status, response_headers)

   return [response_bytes]

def handle_get(environ, start_response):
   status = '200 OK'
   response_json = json.dumps(dict(code=200, data='OK'))
 
   response_bytes = bytes(response_json, encoding = 'utf-8')
 
   data_len = str(len(response_json))
   response_headers = [('Content-Type', 'text/html'),
                  ('Content-Length', data_len)]
   
   start_response(status, response_headers)
   return [response_bytes] 

def handle_post(environ, start_response):
    # the environment variable CONTENT_LENGTH may be empty or missing
   try:
      request_body_size = int(environ.get('CONTENT_LENGTH', 0))
   except (ValueError):
      request_body_size = 0

   request_body = bytes.decode(environ['wsgi.input'].read(request_body_size))
   post_data = parse_qs(request_body)
   
   image_numpy = post_data.get('image_numpy', [''])[0]
   image_numpy = escape(image_numpy)
 
   if len(image_numpy) < 1:
      return response_404(environ, start_response)

   status = '200 OK'
 
   d = Deploy()
 
   response_json = json.dumps(dict(code=200, data=d.f(image_numpy)))
 
   response_bytes = bytes(response_json, encoding = 'utf-8') 
   response_headers = [('Content-Type', 'text/html'),
                  ('Content-Length', str(len(response_json)))]

   start_response(status, response_headers)

   return [response_bytes]
   

def application(environ, start_response):
   method = environ['REQUEST_METHOD']
   path = environ['PATH_INFO']
   if method=='GET' and path == '/_health':
      return handle_get(environ, start_response)
   elif method=='POST' and path == "/":
      return handle_post(environ, start_response)
   else:
      return response_404(environ, start_response)


 
 
class Deploy(metaclass=AutoFill):

    def __init__(self, f=None):
        self.f = f

    @staticmethod
    def register(func):
        Mapper.register(Deploy, func)

    @classmethod
    def run(cls, numworkers=1):
        options = {
            'bind': '%s:%s' % ('0.0.0.0', '2222'),
            'workers': cls.num_of_workers(numworkers),
        }
        StandaloneApplication(application, options).run()

    @classmethod
    def num_of_workers(cls, numworkers=1):
        workers = int(numworkers)
        assert workers > 0, ValueError
        return workers if workers < multiprocessing.cpu_count() * 2 else multiprocessing.cpu_count() * 2






# -*- encoding:utf8 -*-
#================================================================
#   Copyright (C) 2018 Seetatech. All rights reserved.
#   
#   文件名称：setup.py
#   创 建 者： seetatech & xuboxuan
#   创建日期：2018年07月09日
#   描    述：
#
#================================================================

try:
    from setuptools import setup, find_packages, Extension
except ImportError:
    from distutils.core import setup, find_packages
import sys,os

setup(
    name='seetaas-helper',
    version='1.6',
    author='SeetaTech',
    author_email='boxuan.xu@seetatech.com',
    license='MIT',
    platforms='any',
    packages=find_packages(),
    install_requires=[
        'requests==2.18.4',
        'gunicorn',
    ],
    classifiers=[
        'Programming Language :: Python',
        'Operating System :: OS Independent',
    ],
)

